SystemOrganization addCategory: #'Volunti-HomomorphicAuthority-Model'!
SystemOrganization addCategory: #'Volunti-HomomorphicAuthority-View'!

VOAuthority subclass: #VOHomomorphicAuthority
	instanceVariableNames: 'thresholdPublicKey shares paillierPrivateKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-HomomorphicAuthority-Model'!

!VOHomomorphicAuthority commentStamp: 'NicolasVanderAuwera 10/30/2012 07:13' prior: 0!
I implement the homomorphic authority of the voting scheme, Volunti. I use the PaillierThresholdKeyPairGenerator class to generate Paillier key pairs.

Instance Variables:
	thresholdPublicKey	<ProtoObject | PseudoContext>
	shares	<OrderedCollection>
	paillierPrivateKey	<ProtoObject | PseudoContext>!

!VOHomomorphicAuthority methodsFor: 'generator' stamp: 'NicolasVanderAuwera 11/1/2012 14:15'!
generateShares: numberOfVoters
	| ptkpg gen |
	numberOfVoters > 0
		ifFalse: [ ^ self ].
	ptkpg := PaillierThresholdKeyPairGenerator numberOfShares: numberOfVoters * CryptographicParameters maxNumberOfBallots.
	gen := ptkpg setup: CryptographicParameters paillierKeyLength.
	ptkpg keyGen.
	paillierPrivateKey := gen privateKey.
	thresholdPublicKey := ptkpg thresholdPublicKey.
	shares := ptkpg thresholdPrivateShares! !

!VOHomomorphicAuthority methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/5/2012 15:15'!
initialize
	super initialize.
	shares := OrderedCollection new! !

!VOHomomorphicAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:46'!
paillierPrivateKey
	^ paillierPrivateKey! !

!VOHomomorphicAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/6/2012 15:04'!
shares
	^ shares! !

!VOHomomorphicAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:45'!
thresholdPublicKey 
	^ thresholdPublicKey! !

VOAgentComponent subclass: #VOHomomorphicAuthorityComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-HomomorphicAuthority-View'!

!VOHomomorphicAuthorityComponent commentStamp: 'NicolasVanderAuwera 10/30/2012 07:10' prior: 0!
I am the main component of the homomorphic authority.!

!VOHomomorphicAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/7/2012 10:08'!
url
	^ 'homomorphic'! !

!VOHomomorphicAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:38'!
agentClass
	^ VOHomomorphicManager! !

!VOHomomorphicAuthorityComponent methodsFor: 'actions' stamp: 'NicolasVanderAuwera 10/30/2012 06:25'!
setHome
	content := (VOHomomorphicElectionListComponent elections: self agent authorities)
		addDecoration: VOErrorHandlingDecoration new;
		yourself! !

!VOHomomorphicAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/7/2012 10:08'!
title
	^ 'Homomorphic Authority'! !

VOManager subclass: #VOHomomorphicManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-HomomorphicAuthority-Model'!

!VOHomomorphicManager commentStamp: 'NicolasVanderAuwera 10/30/2012 07:12' prior: 0!
I am the homomorphic authority manager.!

!VOHomomorphicManager class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 5/6/2012 15:01'!
organisationUnit
	^ 'Homomorphic Authority'! !

!VOHomomorphicManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/6/2012 15:01'!
port
	^ 9080! !

!VOHomomorphicManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/20/2012 06:35'!
authorityClass
	^ VOHomomorphicAuthority! !

!VOHomomorphicManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:46'!
authorityFor: aProtocol
	^ VOHomomorphicAuthority! !

!VOHomomorphicManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/11/2012 10:09'!
handle: aSymbol
	aSymbol = #shares
		ifTrue: [ self sendShares ].
	aSymbol = #thresholdpublickey
		ifTrue: [ self sendThresholdPublicKey ].
	aSymbol = #paillierprivatekey
		ifTrue: [ self sendPaillierPrivateKey ].
	super handle: aSymbol! !

!VOHomomorphicManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 10/29/2012 15:01'!
sendPaillierPrivateKey
	stream sendObject: self authorityOnStream paillierPrivateKey.
	self addInLog: 'A paillier private key was sent'! !

!VOHomomorphicManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
sendShares
	| subAgent |
	subAgent := self authorityWithId: self idOnStream.
	subAgent generateShares: stream nextLine greaseInteger.
	stream sendObject: subAgent shares! !

!VOHomomorphicManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
sendThresholdPublicKey
	stream sendObject: self authorityOnStream thresholdPublicKey! !

VOElectionsListComponent subclass: #VOHomomorphicElectionListComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-HomomorphicAuthority-View'!

!VOHomomorphicElectionListComponent commentStamp: 'NicolasVanderAuwera 10/30/2012 07:10' prior: 0!
I list all the election and for each of one, the user can download the threshold public key used to decrypt votes.!

!VOHomomorphicElectionListComponent methodsFor: 'actions' stamp: 'NicolasVanderAuwera 10/30/2012 06:36'!
downloadFile: election
	| file |
	file := WAFile new
		contentType: WAMimeType textPlain;
		contents: election thresholdPublicKey xmlDescription;
		fileName: 'threshold_public_key_of_' , election name;
		yourself.
	self requestContext
		respond: [ :response | 
			response
				contentType: 'text/xml';
				document: file rawContents greaseString;
				attachmentWithFileName: file fileName ]! !

!VOHomomorphicElectionListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/30/2012 06:28'!
renderMoreData: election on: html
	html
		tableData: [ 
			html anchor
				class: 'btn btn-info btn-small';
				callback: [ self downloadFile: election ];
				with: [ 
							html span class: 'icon-download'.
							html text: ' Download' ] ]! !

!VOHomomorphicElectionListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/30/2012 06:42'!
renderMoreDataHeadingOn: html
	html tableHeading: 'Paillier threshold public key'! !
